﻿-- addon name and localization table

local a,L=...

-- create frames

local DCL=CreateFrame('frame',a)
local DCLPanel=CreateFrame('frame',a..'Panel')
DCL:RegisterEvent('PLAYER_LOGIN')

-- initialize variables

Decliner_SV={}
local _
local DCLD={}
local DCLD_OnHide
local DCLD_OnShow
local DCL_Filter
local DCL_SetPetitionFrame
local DCL_SetStaticPopup
local DCL_SetTradeFrame
local g='|cff00ff00'
local invrequest
local invtrack={}
local me
local msglist={}
local msgtimer
local notemp=true
local numobj=0
local oldSFX
local player
local plrlist={}
local plrtimer
local r='|cffff0000'
local selfwarn
local soundhandle
local SPD_={}
local temp={}
local tradeinitiated
local vars={}
local defs={
	enabled={duel=true,pet=true,guild=true,group=true,chart=true,trade=true,},
	tell={duel=false,pet=false,guild=false,group=false,chart=false,trade=false,},
	guild={duel=false,pet=false,guild=false,group=false,chart=false,trade=false,},
	friend={duel=false,pet=false,guild=false,group=false,chart=false,trade=false,},
	bnet={duel=false,pet=false,guild=false,group=false,chart=false,trade=false,},
	msg={duel=false,pet=false,guild=false,group=false,chart=false,trade=false,},
	dcl={msgs=false,disable=false,},
}

-- monitored words the player would say to trigger open invites

local txtlist={
	'inv',
	'port',
	'invite',
	'portal',
}

-- the IDs of channels that are monitored. 1=gen, 2=trade, 26=lfg

local chan={
	[1]=true,
	[2]=true,
	[26]=true,
}

-- global string strip
-- originally from Phanx

function DCL:Pattern(str)
	str=gsub(str,'|H.*|h.*|h','') -- removes player links
	str=gsub(str,'%%%d?$?d','') -- removes %d
	str=gsub(str,'%%%d?$?s','') -- removes %s
	str=gsub(str,':.*','') -- removes colons and anything after them
	str=gsub(str,'%(','') -- removes (
	str=gsub(str,'%)','') -- removes )
	str=gsub(str,'%.','') -- removes periods
	str=gsub(str,'^%s','') -- removes leftover space from beginning of string
	str=gsub(str,'%s$','') -- removes leftover space from end of string
	return str
end

-- Blizzard global strings used, with enUS in comments. Parts in quotes are raw strings before DCL:Pattern.
-- this table is only here to keep strings in one place. it's inefficient to do this, especially since some of these are used once.

local S={
	BNF=BATTLENET_OPTIONS_LABEL..' '..FRIENDS,                     --  Battle.net Friends
	CAN=CANCEL,                                                    --  Cancel
	DCA='.*'..DCL:Pattern(ERR_DUEL_CANCELLED)..'.*',               -- "Duel cancelled."
	DDE=DISABLE..' '..a,                                           --  Disable Decliner
	DDM=DISABLE..' '..strupper(ALL)..' '..a..' '..SYSTEM_MESSAGES, --  Disable ALL Decliner System Messages
	DRE='.*'..DCL:Pattern(DUEL_REQUESTED)..'.*',                   -- "%s has challenged you to a duel."
	DUE=DUEL,                                                      --  Duel
	EDE=ENABLE..' '..DECLINE,                                      --  Enable Decline
	GLD=GUILD,                                                     --  Guild
	GME=DCL:Pattern(INSPECT_GUILD_NUM_MEMBERS),                    -- "%d Guild Members"
	GRI='.*'..DCL:Pattern(ERR_INVITED_TO_GROUP_SS)..'.*',          -- "|Hplayer:%s|h[%s]|h has invited you to join a group."
	GRO=GROUP,                                                     --  Group
	GUI='.*'..DCL:Pattern(ERR_INVITED_TO_GUILD_SSS)..'.*',         -- "|Hplayer:%s|h[%s]|h invites you to join %s."
	HID=HIDE,                                                      --  Hide
	INV=DCL:Pattern(GUILDEVENT_TYPE_INVITE),                       -- "%s invites %s"
	OKA=OKAY,                                                      --  Okay
	PBA=PET_BATTLE_PVP_QUEUE,                                      --  Pet Battle
	PBD='.*'..DCL:Pattern(ERR_PETBATTLE_DECLINED)..'.*',           -- "Pet battle invitation declined."
	PBR='.*'..DCL:Pattern(PET_BATTLE_PVP_DUEL_REQUESTED)..'.*',    -- "%s has challenged you to a pet-battle."
	PTN=DCL:Pattern(PETITION_TITLE),                               -- "Petition: %s"
	REA=strlower(READY),                                           --  Ready
	REQ=GUILDINFOTAB_APPLICANTS_NONE,                              --  Requests
	RFR=DCL:Pattern(FRIENDS_LIST_REALM)..' '..FRIENDS,             -- "Realm: %s" Friends
	SME=SYSTEM_MESSAGES,                                           --  System Messages
	TCA='.*'..DCL:Pattern(ERR_TRADE_CANCELLED)..'.*',              -- "Trade cancelled."
	TIN='.*'..DCL:Pattern(ERR_INITIATE_TRADE_S)..'.*',             -- "You have requested to trade with %s."
	TRA=TRADE,                                                     --  Trade
	VER=strlower(GAME_VERSION_LABEL),                              --  Version
	WHI=DCL:Pattern(AUTOCOMPLETE_LABEL_INTERACTED),                -- "(Whispered)"
}

-- timer to control open invites, chat messages, and list of whispered players

function DCL:Timer(str)
	if str == 'inv' then
		tinsert(invtrack,'track')
		C_Timer.After(300,function()
			if #invtrack == 1 then
				invrequest=false
				invtrack={}
			else
				tremove(invtrack,1)
			end
		end)
	elseif str == 'msg' then
		msgtimer=true
		C_Timer.After(60,function()
			tremove(msglist,1)
			if next(msglist) ~= nil then
				DCL:Timer('msg')
			else
				msgtimer=false
			end
		end)
	elseif str == 'plr' then
		plrtimer=true
		C_Timer.After(60,function()
			tremove(plrlist,1)
			if next(plrlist) ~= nil then
				DCL:Timer('plr')
			else
				plrtimer=false
			end
		end)
	end
end

-- chat output
-- the idea for faking a yellow system message came from Jaliborc's Scrap
-- but they hook into the MessageHandler function while I use AddMessage

function DCL:Print(msg)
	if vars.dcl.msgs then return end
	if next(msglist) ~= nil then
		for k,v in next,msglist do
			if msg == v then return end
		end
	end
	if not selfwarn then tinsert(msglist,msg) end selfwarn=false
	if not msgtimer then DCL:Timer('msg') end
	local info,notfound=ChatTypeInfo['SYSTEM'],true
	for i=1,NUM_CHAT_WINDOWS do
		local chatframe=_G['ChatFrame'..i]
		if chatframe:IsEventRegistered('CHAT_MSG_SYSTEM') then
			notfound=false
			chatframe:AddMessage(r..a..': |r'..msg,info.r,info.g,info.b,info.id)
		end
	end
	if notfound then
		DEFAULT_CHAT_FRAME:AddMessage(r..a..': |r'..msg,info.r,info.g,info.b,info.id)
	end
end

-- main bouncer, which ultimately decides if a player is declined

function DCL:IsAllowed(var)
	if vars.dcl.disable or not vars.enabled[var] then return true end
	if not vars.tell[var] then
		if next(plrlist) ~= nil then
			for _,v in next,plrlist do
				if player == v then
				return true end
			end
		end
	end
	if var ~= 'guild' and IsInGuild() and not vars.guild[var] then
		for i=1,(GetNumGuildMembers()) do
			if player == Ambiguate((GetGuildRosterInfo(i)),'short') then
			return true end
		end
	end
	if not vars.friend[var] then
		for i=1,(GetNumFriends()) do
			if player == Ambiguate((GetFriendInfo(i)),'short') then
			return true end
		end
	end
	if not vars.bnet[var] then
		for i=1,(BNGetNumFriends()) do
			for j=1,(BNGetNumFriendToons(i)) do
				local _,name=BNGetFriendToonInfo(i,j)
				if player == Ambiguate(name,'short') then
				return true end
			end
		end
	end
	return false
end

-- allows all group invites 5 minutes after player last said certain words
-- changing zones or joining a group will cancel this effect
-- being in a group will not trigger this effect

function DCL:OpenInvite(msg,char)
	if IsInGroup() or char ~= me then return end
	for _,v in next,txtlist do
		if strmatch(msg,'^'..v..'[^%w].*')
		or strmatch(msg,'^'..v..'$')
		or strmatch(msg,'.*[^%w]'..v..'$')
		or strmatch(msg,'^.*[^%w]'..v..'[^%w].*$') then
			invrequest=true
			DCL:Timer('inv')
		end
	end
end

-- following function from http://lua-users.org/wiki/CopyTable
-- this copies the values of a table into another table instead of creating a reference

local function tcopy(src)
	local copy
	if type(src) == 'table' then
		copy={}
		for k,v in next,src do
			copy[tcopy(k)]=tcopy(v)
		end
		setmetatable(copy,tcopy(getmetatable(src)))
	else
		copy=src
	end
	return copy
end

-- event blocks

function DCL:PLAYER_LOGIN()
	local name,realm=UnitFullName('player')
	me=name..'-'..realm
	GuildRoster()
	ShowFriends()
	oldSFX=GetCVar('Sound_EnableSFX')
	SPD_['DUEL_REQUESTED']=tcopy(StaticPopupDialogs['DUEL_REQUESTED'])
	SPD_['PET_BATTLE_PVP_DUEL_REQUESTED']=tcopy(StaticPopupDialogs['PET_BATTLE_PVP_DUEL_REQUESTED'])
	SPD_['PARTY_INVITE']=tcopy(StaticPopupDialogs['PARTY_INVITE'])
	SPD_['PARTY_INVITE_XREALM']=tcopy(StaticPopupDialogs['PARTY_INVITE_XREALM'])
	if Decliner_SV.check then
		for k1,v1 in next,defs do
			if Decliner_SV[k1] == nil then
				Decliner_SV[k1]=defs[k1]
				vars[k1]=defs[k1]
			else
				vars[k1]=Decliner_SV[k1]
			end
			for k2 in next,v1 do
				if Decliner_SV[k1][k2] == nil then
					Decliner_SV[k1][k2]=defs[k1][k2]
					vars[k1][k2]=defs[k1][k2]
				elseif type(Decliner_SV[k1][k2]) ~= 'boolean' then
					Decliner_SV[k1][k2]=defs[k1][k2]
					vars[k1][k2]=defs[k1][k2]
				else
					vars[k1][k2]=Decliner_SV[k1][k2]
				end
			end
		end
		Decliner_SV=nil
		Decliner_SV=tcopy(vars)
		Decliner_SV.check=true
	else
		vars=tcopy(defs)
		Decliner_SV=nil
		Decliner_SV=tcopy(defs)
		Decliner_SV.check=true
	end
	DCL:BlizzCode()
	DCL:BTBGI()
	DCL:Options()
	DCL:Register()
	DCL:Print(S['VER']..' '..GetAddOnMetadata(a,'Version')..' '..S['REA'])
	DCL:UnregisterEvent('PLAYER_LOGIN')
end

function DCL:CHAT_MSG_CHANNEL(msg,char,_,_,_,_,ID)
	if chan[ID] then DCL:OpenInvite(msg,char) else return end
end

function DCL:CHAT_MSG_EMOTE(msg,char)
	DCL:OpenInvite(msg,char)
end

function DCL:CHAT_MSG_SAY(msg,char)
	DCL:OpenInvite(msg,char)
end

function DCL:CHAT_MSG_YELL(msg,char)
	DCL:OpenInvite(msg,char)
end

function DCL:GROUP_ROSTER_UPDATE()
	invrequest=false
end

function DCL:ZONE_CHANGED_NEW_AREA()
	invrequest=false
end

function DCL:CHAT_MSG_SYSTEM(msg)
	if strmatch(msg,S['TIN']) then tradeinitiated=true end
end

function DCL:CHAT_MSG_WHISPER_INFORM(_,name)
	tinsert(plrlist,Ambiguate(name,'short'))
	if not plrtimer then DCL:Timer('plr') end
end

function DCL:CVAR_UPDATE(cvar,value)
	oldSFX=GetCVar('Sound_EnableSFX')
	if cvar == 'BLOCK_TRADES' then
		if value == '0' and not vars.enabled.trade then
			vars.enabled.trade=true
			DCL:Register()
			DCL:SetCheckBoxes()
			DCL:BTBGI()
			selfwarn=true
			DCL:Print(L['X_BLOCK_TRADES_OFF'])
		end
		if value == '1' and vars.enabled.trade then
			vars.enabled.trade=false
			DCL:Register()
			DCL:SetCheckBoxes()
			DCL:BTBGI()
			selfwarn=true
			DCL:Print(L['X_BLOCK_TRADES_ON'])
		end
	end
end

function DCL:DISABLE_DECLINE_GUILD_INVITE()
	if vars.enabled.guild then return end
	vars.enabled.guild=true
	DCL:Register()
	DCL:SetCheckBoxes()
	DCL:BTBGI()
	selfwarn=true
	DCL:Print(L['X_BLOCK_GUILDS_OFF'])
end

function DCL:ENABLE_DECLINE_GUILD_INVITE()
	if not vars.enabled.guild then return end
	vars.enabled.guild=false
	DCL:Register()
	DCL:SetCheckBoxes()
	DCL:BTBGI()
	selfwarn=true
	DCL:Print(L['X_BLOCK_GUILDS_ON'])
end

function DCL:DUEL_REQUESTED(name)
	local _,frame=StaticPopup_Visible('DUEL_REQUESTED')
	HideUIPanel(frame)
	player=Ambiguate(name,'short')
	if DCL:IsAllowed('duel') then
		DCL_SetStaticPopup(frame,'default')
		ShowUIPanel(frame)
		PlaySoundKitID(880)
	else
		CancelDuel()
		if not vars.msg.duel then
			DCL:Print(format(L['DECLINED_DUEL'],'|Hplayer:'..name..'|h['..name..']|h'))
		end
	end
end

function DCL:PET_BATTLE_PVP_DUEL_REQUESTED(name)
	local _,frame=StaticPopup_Visible('PET_BATTLE_PVP_DUEL_REQUESTED')
	HideUIPanel(frame)
	player=Ambiguate(name,'short')
	if DCL:IsAllowed('pet') then
		DCL_SetStaticPopup(frame,'default')
		ShowUIPanel(frame)
		PlaySoundKitID(880)
	else
		C_PetBattles.CancelPVPDuel()
		if not vars.msg.pet then
			DCL:Print(format(L['DECLINED_PET'],'|Hplayer:'..name..'|h['..name..']|h'))
		end
	end
end

function DCL:GUILD_INVITE_REQUEST(name,guild)
	SetCVar('Sound_EnableSFX','0')
	player=Ambiguate(name,'short')
	if DCL:IsAllowed('guild') then
		C_Timer.After(0.5,function()
			SetCVar('Sound_EnableSFX',oldSFX)
			PlaySoundKidID(888)
		end)
	else
		StaticPopupSpecial_Hide(GuildInviteFrame)
		DeclineGuild()
		if not vars.msg.guild then
			if GetLocale() == 'koKR' or GetLocale() == 'zhCN' or GetLocale() == 'zhTW' then
				DCL:Print(format(L['DECLINED_GUILD'],guild,'|Hplayer:'..name..'|h['..name..']|h'))
			else
				DCL:Print(format(L['DECLINED_GUILD'],'|Hplayer:'..name..'|h['..name..']|h',guild))
			end
		end
		C_Timer.After(0.5,function()SetCVar('Sound_EnableSFX',oldSFX)end)
	end
end

function DCL:PARTY_INVITE_REQUEST(name)
	local which='PARTY_INVITE_XREALM'
	local _,frame=StaticPopup_Visible(which)
	if not frame then
		which='PARTY_INVITE'
		_,frame=StaticPopup_Visible(which)
	end
	if frame then HideUIPanel(frame) end
	player=Ambiguate(name,'short')
	if DCL:IsAllowed('group') or invrequest==true then
		if frame then
			DCL_SetStaticPopup(frame,'default')
			ShowUIPanel(frame)
			PlaySoundKitID(880)
			StaticPopupDialogs[which].OnHide=function(self)
				if not self.inviteAccepted then
					DeclineGroup()
					self:Hide()
				end
				StaticPopupDialogs[which]=tcopy(DCLD[which])
			end
		end
	else
		DeclineGroup()
		if not vars.msg.group then
			DCL:Print(format(L['DECLINED_GROUP'],'|Hplayer:'..name..'|h['..name..']|h'))
		end
	end
end

function DCL:PETITION_SHOW()
	HideUIPanel(PetitionFrame)
	local ptype,_,_,_,name,originator=GetPetitionInfo()
	player=Ambiguate(name,'short')
	if originator or DCL:IsAllowed('chart') then
		DCL_SetPetitionFrame('default')
		ShowUIPanel(PetitionFrame)
	else
		ClosePetition()
		if not vars.msg.chart then
			if ptype == 'guild' then
				DCL:Print(format(L['DECLINED_CHART_GUILD'],'|Hplayer:'..name..'|h['..name..']|h'))
			else
				DCL:Print(format(L['DECLINED_CHART_NIL'],'|Hplayer:'..name..'|h['..name..']|h'))
			end
		end
	end
end

function DCL:TRADE_SHOW()
	HideUIPanel(TradeFrame)
	local name=(UnitName('npc')) -- thank you to http://wowprogramming.com/docs/api_types#unitID for that vague Trade UI note next to 'npc'
	player=Ambiguate(name,'short')
	if tradeinitiated or DCL:IsAllowed('trade') then
		DCL_SetTradeFrame('default')
		ShowUIPanel(TradeFrame)
	else
		CloseTrade()
		if not vars.msg.trade then
			DCL:Print(format(L['DECLINED_TRADE'],'|Hplayer:'..name..'|h['..name..']|h'))
		end
	end
end

function DCL:TRADE_CLOSED()
	tradeinitiated=false
end

function DCL:UI_INFO_MESSAGE(msg)
	if strmatch(msg,S['DCA']) and not DCL:IsAllowed('duel') then UIErrorsFrame:Clear() end
	if strmatch(msg,S['PBD']) and not DCL:IsAllowed('pet') then UIErrorsFrame:Clear() end
	if strmatch(msg,S['TCA']) and not DCL:IsAllowed('trade') then UIErrorsFrame:Clear() end
end

-- magic line of code that calls event blocks when their namesakes trigger

DCL:SetScript('OnEvent',function(self,event,...)self[event](self,...)end)

-- all Blizzard code wrapped in one function

function DCL:BlizzCode()

-- chat filter that blocks associated system messages if that action is declined

	DCL_Filter=function(_,_,msg)
		if vars.dcl.disable then return false end
		if strmatch(msg,S['DRE']) and not DCL:IsAllowed('duel') then return true end
		if strmatch(msg,S['PBR']) and not DCL:IsAllowed('pet') then return true end
		if strmatch(msg,S['GUI']) and not DCL:IsAllowed('guild') then return true end
		if strmatch(msg,S['GRI']) and not DCL:IsAllowed('group') then return true end
		return false
	end
	ChatFrame_AddMessageEventFilter('CHAT_MSG_SYSTEM',DCL_Filter)

-- begin Blizzard code replacements (these are injected into the UI, so direct usage of global strings instead of S table)
-- all of these are basically alterations of Blizzard's Lua or changing Blizzard's frame scripts
-- you can view Blizzard's Lua with these instructions: http://wow.gamepedia.com/Viewing_Blizzard's_interface_code

	DCLD['DUEL_REQUESTED']={
		text=DUEL_REQUESTED,
		button1=ACCEPT,
		button2=DECLINE,
		OnAccept=function(self)
			AcceptDuel()
		end,
		OnCancel=function(self)
			CancelDuel()
		end,
		timeout=STATICPOPUP_TIMEOUT,
		hideOnEscape=1
	}

	DCLD['PET_BATTLE_PVP_DUEL_REQUESTED']={
		text=PET_BATTLE_PVP_DUEL_REQUESTED,
		button1=ACCEPT,
		button2=DECLINE,
		OnAccept=function(self)
			C_PetBattles.AcceptPVPDuel()
		end,
		OnCancel=function(self)
			C_PetBattles.CancelPVPDuel()
		end,
		timeout=STATICPOPUP_TIMEOUT,
		hideOnEscape=1
	}

	DCLD['PARTY_INVITE']={
		text=INVITATION,
		button1=ACCEPT,
		button2=DECLINE,
		OnShow=function(self)
			self.inviteAccepted=nil
		end,
		OnAccept=function(self)
			AcceptGroup()
			self.inviteAccepted=1
		end,
		OnCancel=function(self)
			DeclineGroup()
		end,
		OnHide=function(self)
			if (not self.inviteAccepted) then
				self:Hide()
			end
		end,
		timeout=STATICPOPUP_TIMEOUT,
		whileDead=1,
		hideOnEscape=1
	}

	DCLD['PARTY_INVITE_XREALM']={
		text=INVITATION_XREALM,
		button1=ACCEPT,
		button2=DECLINE,
		OnShow=function(self)
			self.inviteAccepted=nil
		end,
		OnAccept=function(self)
			AcceptGroup()
			self.inviteAccepted=1
		end,
		OnCancel=function(self)
			DeclineGroup()
		end,
		OnHide=function(self)
			if (not self.inviteAccepted) then
				self:Hide()
			end
		end,
		timeout=STATICPOPUP_TIMEOUT,
		whileDead=1,
		hideOnEscape=1
	}

	DCLD_OnShow=function(frame)
		local dialog=frame.which
		if dialog and
		((dialog == 'DUEL_REQUESTED' and vars.enabled.duel) or
		(dialog == 'PET_BATTLE_PVP_DUEL_REQUESTED' and vars.enabled.pet) or
		((dialog == 'PARTY_INVITE' or dialog == 'PARTY_INVITE_XREALM') and vars.enabled.group)) then
			local OnShow=dialog.OnShow
			if (OnShow) then
				OnShow(frame,frame.data)
			end
			if (dialog.enterClicksFirstButton) then
				frame:SetScript('OnKeyDown',StaticPopup_OnKeyDown)
			end
		else
			StaticPopup_OnShow(frame)
		end
	end

	DCLD_OnHide=function(frame)
		local dialog=frame.which
		if dialog and
		((dialog == 'DUEL_REQUESTED' and vars.enabled.duel) or
		(dialog == 'PET_BATTLE_PVP_DUEL_REQUESTED' and vars.enabled.pet) or
		((dialog == 'PARTY_INVITE' or dialog == 'PARTY_INVITE_XREALM') and vars.enabled.group)) then
			StaticPopup_CollapseTable()
			local OnHide=dialog.OnHide
			if (OnHide) then
				OnHide(frame,frame.data)
			end
			if (dialog.enterClicksFirstButton) then
				frame:SetScript('OnKeyDown',nil)
			end
		else
			StaticPopup_OnHide(frame)
		end
	end

	DCL_SetStaticPopup=function(frame,action)
		frame:SetScript('OnShow',nil)
		frame:SetScript('OnHide',nil)
		if action == 'decline' then
			frame:SetScript('OnShow',function()
				DCLD_OnShow(frame)
			end)
			frame:SetScript('OnHide',function()
				DCLD_OnHide(frame)
			end)
		elseif action == 'default' and vars.dcl.disable then
			frame:SetScript('OnShow',function()
				StaticPopup_OnShow(frame)
			end)
			frame:SetScript('OnHide',function()
				StaticPopup_OnHide(frame)
			end)
		elseif action == 'default' then
			frame:SetScript('OnShow',function()
				StaticPopup_OnShow(frame)
			end)
			frame:SetScript('OnHide',function()
				StaticPopup_OnHide(frame)
				if vars.enabled.duel or vars.enabled.pet or vars.enabled.group then
					DCL_SetStaticPopup(frame,'decline')
				end
			end)
		end
	end

	DCL_SetPetitionFrame=function(action)
		PetitionFrame:SetScript('OnShow',nil)
		PetitionFrame:SetScript('OnHide',nil)
		if action == 'decline' then
			PetitionFrame:SetScript('OnHide',function()
				StaticPopup_Hide('CONFIRM_GUILD_CHARTER_SIGNATURE')
			end)
		elseif action == 'default' and vars.dcl.disable then
			PetitionFrame:SetScript('OnShow',function()
				PlaySoundKitID(875)
			end)
			PetitionFrame:SetScript('OnHide',function()
				PlaySoundKitID(876)
				StaticPopup_Hide('CONFIRM_GUILD_CHARTER_SIGNATURE')
				ClosePetition()
			end)
		elseif action == 'default' then
			PetitionFrame:SetScript('OnShow',function()
				PlaySoundKitID(875)
			end)
			PetitionFrame:SetScript('OnHide',function()
				PlaySoundKitID(876)
				StaticPopup_Hide('CONFIRM_GUILD_CHARTER_SIGNATURE')
				ClosePetition()
				if vars.enabled.chart then DCL_SetPetitionFrame('decline') end
			end)
		end
	end

	DCL_SetTradeFrame=function(action)
		TradeFrame:SetScript('OnShow',nil)
		TradeFrame:SetScript('OnHide',nil)
		if action == 'decline' then
			TradeFrame:SetScript('OnShow',function()
				TradeFrame_OnShow(self)
			end)
			TradeFrame:SetScript('OnHide',function()
				MoneyInputFrame_SetCopper(TradePlayerInputMoneyFrame,0)
			end)
		elseif action == 'default' and vars.dcl.disable then
			TradeFrame:SetScript('OnShow',function()
				TradeFrame_OnShow(self)
				PlaySoundKitID(839)
			end)
			TradeFrame:SetScript('OnHide',function()
				TradeFrame_OnHide()
				PlaySoundKitID(840)
			end)
		elseif action == 'default' then
			TradeFrame:SetScript('OnShow',function()
				TradeFrame_OnShow(self)
				PlaySoundKitID(839)
			end)
			TradeFrame:SetScript('OnHide',function()
				TradeFrame_OnHide()
				PlaySoundKitID(840)
				if vars.enabled.trade then DCL_SetTradeFrame('decline') end
			end)
		end
	end

end

-- event registration

function DCL:Register()
	if vars.dcl.disable then
		DCL:UnregisterAllEvents()
		StaticPopupDialogs['DUEL_REQUESTED']=tcopy(SPD_['DUEL_REQUESTED'])
		StaticPopupDialogs['PET_BATTLE_PVP_DUEL_REQUESTED']=tcopy(SPD_['PET_BATTLE_PVP_DUEL_REQUESTED'])
		StaticPopupDialogs['PARTY_INVITE']=tcopy(SPD_['PARTY_INVITE'])
		StaticPopupDialogs['PARTY_INVITE_XREALM']=tcopy(SPD_['PARTY_INVITE_XREALM'])
		DCL_SetPetitionFrame('default')
		DCL_SetTradeFrame('default')
		for i=1,STATICPOPUP_NUMDIALOGS do
			local frame=_G['StaticPopup'..i]
			DCL_SetStaticPopup(frame,'default')
		end
	else
		DCL:RegisterEvent('CHAT_MSG_CHANNEL')
		DCL:RegisterEvent('CHAT_MSG_EMOTE')
		DCL:RegisterEvent('CHAT_MSG_SAY')
		DCL:RegisterEvent('CHAT_MSG_YELL')
		DCL:RegisterEvent('GROUP_ROSTER_UPDATE')
		DCL:RegisterEvent('ZONE_CHANGED_NEW_AREA')
		DCL:RegisterEvent('CHAT_MSG_SYSTEM')
		DCL:RegisterEvent('CHAT_MSG_WHISPER_INFORM')
		DCL:RegisterEvent('CVAR_UPDATE')
		DCL:RegisterEvent('DISABLE_DECLINE_GUILD_INVITE')
		DCL:RegisterEvent('ENABLE_DECLINE_GUILD_INVITE')
		if vars.enabled.duel then
			DCL:RegisterEvent('DUEL_REQUESTED')
			StaticPopupDialogs['DUEL_REQUESTED']=tcopy(DCLD['DUEL_REQUESTED'])
		else
			DCL:UnregisterEvent('DUEL_REQUESTED')
			StaticPopupDialogs['DUEL_REQUESTED']=tcopy(SPD_['DUEL_REQUESTED'])
		end
		if vars.enabled.pet then
			DCL:RegisterEvent('PET_BATTLE_PVP_DUEL_REQUESTED')
			StaticPopupDialogs['PET_BATTLE_PVP_DUEL_REQUESTED']=tcopy(DCLD['PET_BATTLE_PVP_DUEL_REQUESTED'])
		else
			DCL:UnregisterEvent('PET_BATTLE_PVP_DUEL_REQUESTED')
			StaticPopupDialogs['PET_BATTLE_PVP_DUEL_REQUESTED']=tcopy(SPD_['PET_BATTLE_PVP_DUEL_REQUESTED'])
		end
		if vars.enabled.guild then
			DCL:RegisterEvent('GUILD_INVITE_REQUEST')
		else
			DCL:UnregisterEvent('GUILD_INVITE_REQUEST')
		end
		if vars.enabled.group then
			DCL:RegisterEvent('PARTY_INVITE_REQUEST')
			StaticPopupDialogs['PARTY_INVITE']=tcopy(DCLD['PARTY_INVITE'])
			StaticPopupDialogs['PARTY_INVITE_XREALM']=tcopy(DCLD['PARTY_INVITE_XREALM'])
		else
			DCL:UnregisterEvent('PARTY_INVITE_REQUEST')
			StaticPopupDialogs['PARTY_INVITE']=tcopy(SPD_['PARTY_INVITE'])
			StaticPopupDialogs['PARTY_INVITE_XREALM']=tcopy(SPD_['PARTY_INVITE_XREALM'])
		end
		if vars.enabled.chart then
			DCL:RegisterEvent('PETITION_SHOW')
			DCL_SetPetitionFrame('decline')
		else
			DCL:UnregisterEvent('PETITION_SHOW')
			DCL_SetPetitionFrame('default')
		end
		if vars.enabled.trade then
			DCL:RegisterEvent('TRADE_SHOW')
			DCL:RegisterEvent('TRADE_CLOSED')
			DCL_SetTradeFrame('decline')
		else
			DCL:UnregisterEvent('TRADE_SHOW')
			DCL:UnregisterEvent('TRADE_CLOSED')
			DCL_SetTradeFrame('default')
		end
		if vars.enabled.duel or vars.enabled.pet or vars.enabled.group then
			for i=1,STATICPOPUP_NUMDIALOGS do
				local frame=_G['StaticPopup'..i]
				DCL_SetStaticPopup(frame,'decline')
			end
		else
			for i=1,STATICPOPUP_NUMDIALOGS do
				local frame=_G['StaticPopup'..i]
				DCL_SetStaticPopup(frame,'default')
			end
		end
		if vars.enabled.duel or vars.enabled.pet or vars.enabled.trade then
			DCL:RegisterEvent('UI_INFO_MESSAGE')
		else
			DCL:UnregisterEvent('UI_INFO_MESSAGE')
		end
	end
end

-- disables the block trades and block guild invites options and edits their display text based on user options
-- Decliner won't be able to let authorized players through if these options are enabled

function DCL:BTBGI()
	if vars.enabled.guild and not vars.dcl.disable then
		_G['InterfaceOptionsControlsPanelBlockGuildInvitesText']:SetText('|cff666666'..BLOCK_GUILD_INVITES..' ('..a..')')
		SetAutoDeclineGuildInvites(false)
	else
		_G['InterfaceOptionsControlsPanelBlockGuildInvitesText']:SetText(BLOCK_GUILD_INVITES)
	end
	if vars.enabled.trade and not vars.dcl.disable then
		_G['InterfaceOptionsControlsPanelBlockTradesText']:SetText('|cff666666'..BLOCK_TRADES..' ('..a..')')
		SetCVar('blockTrades','0')
	else
		_G['InterfaceOptionsControlsPanelBlockTradesText']:SetText(BLOCK_TRADES)
	end
end

-- Interface Options panel
-- I would like to thank FlexYourHead of the addon Work_Complete for a ton of comments in their code
-- It made understanding buttons, text, and the options frame a whole lot easier

function DCL:Create(obj,xloc,yloc,misc,text)
	numobj=numobj+1
	local name=a..'Object'..format('%02i',numobj)
	if obj == 'b' then
		obj=CreateFrame('CheckButton',name,DCLPanel,'InterfaceOptionsCheckButtonTemplate')
		obj:SetPoint('CENTER',DCLPanel,'TOPLEFT',xloc,yloc)
		obj:SetHitRectInsets(0,0,0,0)
		obj.tooltipText=misc
		_G[name..'Text']:SetText('|cffffd100'..text)
	elseif obj == 't' then
		obj=DCLPanel:CreateFontString(name,'ARTWORK','GameFontNormalSmall')
		obj:SetPoint(misc,DCLPanel,'TOPLEFT',xloc,yloc)
		obj:SetText(text)
	end
	return obj
end

function DCL:Options()
	DCLPanel.name=a
	InterfaceOptions_AddCategory(DCLPanel,a)
	DCLtext01=DCL:Create('t',144,-32,'CENTER',S['DUE'])
	DCLtext02=DCL:Create('t',144,-48,'CENTER',S['REQ'])
	DCLtext03=DCL:Create('t',208,-32,'CENTER',S['PBA'])
	DCLtext04=DCL:Create('t',208,-48,'CENTER',S['REQ'])
	DCLtext05=DCL:Create('t',272,-32,'CENTER',S['GLD'])
	DCLtext06=DCL:Create('t',272,-48,'CENTER',S['INV'])
	DCLtext07=DCL:Create('t',336,-32,'CENTER',S['GRO'])
	DCLtext08=DCL:Create('t',336,-48,'CENTER',S['INV'])
	DCLtext09=DCL:Create('t',400,-32,'CENTER',S['GLD'])
	DCLtext10=DCL:Create('t',400,-48,'CENTER',S['PTN'])
	DCLtext11=DCL:Create('t',464,-32,'CENTER',S['TRA'])
	DCLtext12=DCL:Create('t',464,-48,'CENTER',S['REQ'])
	DCLtext13=DCL:Create('t',8,-80,'LEFT',S['EDE'])
	DCLbox01=DCL:Create('b',144,-80,g..L['ENABLE_DUEL'],'')
	DCLbox02=DCL:Create('b',208,-80,g..L['ENABLE_PET'],'')
	DCLbox03=DCL:Create('b',272,-80,g..L['ENABLE_GUILD'],'')
	DCLbox04=DCL:Create('b',336,-80,g..L['ENABLE_GROUP'],'')
	DCLbox05=DCL:Create('b',400,-80,g..L['ENABLE_CHART'],'')
	DCLbox06=DCL:Create('b',464,-80,g..L['ENABLE_TRADE'],'')
	DCLtext14=DCL:Create('t',8,-144,'LEFT',S['WHI'])
	DCLbox07=DCL:Create('b',144,-144,r..L['TELL_DUEL'],'')
	DCLbox08=DCL:Create('b',208,-144,r..L['TELL_PET'],'')
	DCLbox09=DCL:Create('b',272,-144,r..L['TELL_GUILD'],'')
	DCLbox10=DCL:Create('b',336,-144,r..L['TELL_GROUP'],'')
	DCLbox11=DCL:Create('b',400,-144,r..L['TELL_CHART'],'')
	DCLbox12=DCL:Create('b',464,-144,r..L['TELL_TRADE'],'')
	DCLtext15=DCL:Create('t',8,-208,'LEFT',S['GME'])
	DCLbox13=DCL:Create('b',144,-208,r..L['GUILD_DUEL'],'')
	DCLbox14=DCL:Create('b',208,-208,r..L['GUILD_PET'],'')
	DCLbox15=DCL:Create('b',272,-208,r..L['GUILD_GUILD'],'')
	DCLbox16=DCL:Create('b',336,-208,r..L['GUILD_GROUP'],'')
	DCLbox17=DCL:Create('b',400,-208,r..L['GUILD_CHART'],'')
	DCLbox18=DCL:Create('b',464,-208,r..L['GUILD_TRADE'],'')
	DCLtext16=DCL:Create('t',8,-272,'LEFT',S['RFR'])
	DCLbox19=DCL:Create('b',144,-272,r..L['FRIEND_DUEL'],'')
	DCLbox20=DCL:Create('b',208,-272,r..L['FRIEND_PET'],'')
	DCLbox21=DCL:Create('b',272,-272,r..L['FRIEND_GUILD'],'')
	DCLbox22=DCL:Create('b',336,-272,r..L['FRIEND_GROUP'],'')
	DCLbox23=DCL:Create('b',400,-272,r..L['FRIEND_CHART'],'')
	DCLbox24=DCL:Create('b',464,-272,r..L['FRIEND_TRADE'],'')
	DCLtext17=DCL:Create('t',8,-336,'LEFT',S['BNF'])
	DCLbox25=DCL:Create('b',144,-336,r..L['BNET_DUEL'],'')
	DCLbox26=DCL:Create('b',208,-336,r..L['BNET_PET'],'')
	DCLbox27=DCL:Create('b',272,-336,r..L['BNET_GUILD'],'')
	DCLbox28=DCL:Create('b',336,-336,r..L['BNET_GROUP'],'')
	DCLbox29=DCL:Create('b',400,-336,r..L['BNET_CHART'],'')
	DCLbox30=DCL:Create('b',464,-336,r..L['BNET_TRADE'],'')
	DCLtext18=DCL:Create('t',8,-400,'LEFT',S['SME'])
	DCLbox31=DCL:Create('b',144,-400,r..S['HID']..' "'..a..':'..S['DUE']..'" '..S['SME'],'')
	DCLbox32=DCL:Create('b',208,-400,r..S['HID']..' "'..a..':'..S['PBA']..'" '..S['SME'],'')
	DCLbox33=DCL:Create('b',272,-400,r..S['HID']..' "'..a..':'..S['GLD']..'" '..S['SME'],'')
	DCLbox34=DCL:Create('b',336,-400,r..S['HID']..' "'..a..':'..S['GRO']..'" '..S['SME'],'')
	DCLbox35=DCL:Create('b',400,-400,r..S['HID']..' "'..a..':'..S['PTN']..'" '..S['SME'],'')
	DCLbox36=DCL:Create('b',464,-400,r..S['HID']..' "'..a..':'..S['TRA']..'" '..S['SME'],'')
	DCLbox37=DCL:Create('b',144,-464,r..L['X_DISABLE_MESSAGES'],S['DDM'])
	DCLbox38=DCL:Create('b',144,-496,r..L['X_DISABLE_DECLINER'],S['DDE'])
	DCLbox01:SetScript('PostClick',function(self)vars.enabled.duel=self:GetChecked()DCL:Register()end)
	DCLbox02:SetScript('PostClick',function(self)vars.enabled.pet=self:GetChecked()DCL:Register()end)
	DCLbox03:SetScript('PostClick',function(self)vars.enabled.guild=self:GetChecked()DCL:Register()DCL:BTBGI()end)
	DCLbox04:SetScript('PostClick',function(self)vars.enabled.group=self:GetChecked()DCL:Register()end)
	DCLbox05:SetScript('PostClick',function(self)vars.enabled.chart=self:GetChecked()DCL:Register()end)
	DCLbox06:SetScript('PostClick',function(self)vars.enabled.trade=self:GetChecked()DCL:Register()DCL:BTBGI()end)
	DCLbox07:SetScript('PostClick',function(self)vars.tell.duel=self:GetChecked()DCL:Register()end)
	DCLbox08:SetScript('PostClick',function(self)vars.tell.pet=self:GetChecked()DCL:Register()end)
	DCLbox09:SetScript('PostClick',function(self)vars.tell.guild=self:GetChecked()DCL:Register()end)
	DCLbox10:SetScript('PostClick',function(self)vars.tell.group=self:GetChecked()DCL:Register()end)
	DCLbox11:SetScript('PostClick',function(self)vars.tell.chart=self:GetChecked()DCL:Register()end)
	DCLbox12:SetScript('PostClick',function(self)vars.tell.trade=self:GetChecked()DCL:Register()end)
	DCLbox13:SetScript('PostClick',function(self)vars.guild.duel=self:GetChecked()DCL:Register()end)
	DCLbox14:SetScript('PostClick',function(self)vars.guild.pet=self:GetChecked()DCL:Register()end)

-- fun little function for a useless "guild invites from guild members" setting that plays a random sound/voice/music file from the entire game
-- this is here so that there isn't a blank spot in the grid of checkboxes

	DCLbox15:SetScript('PostClick',function(self)if soundhandle then StopSound(soundhandle)end local x=random(1,51000)_,soundhandle=PlaySoundKitID(x)self:SetChecked(false)end)

	DCLbox16:SetScript('PostClick',function(self)vars.guild.group=self:GetChecked()DCL:Register()end)
	DCLbox17:SetScript('PostClick',function(self)vars.guild.chart=self:GetChecked()DCL:Register()end)
	DCLbox18:SetScript('PostClick',function(self)vars.guild.trade=self:GetChecked()DCL:Register()end)
	DCLbox19:SetScript('PostClick',function(self)vars.friend.duel=self:GetChecked()DCL:Register()end)
	DCLbox20:SetScript('PostClick',function(self)vars.friend.pet=self:GetChecked()DCL:Register()end)
	DCLbox21:SetScript('PostClick',function(self)vars.friend.guild=self:GetChecked()DCL:Register()end)
	DCLbox22:SetScript('PostClick',function(self)vars.friend.group=self:GetChecked()DCL:Register()end)
	DCLbox23:SetScript('PostClick',function(self)vars.friend.chart=self:GetChecked()DCL:Register()end)
	DCLbox24:SetScript('PostClick',function(self)vars.friend.trade=self:GetChecked()DCL:Register()end)
	DCLbox25:SetScript('PostClick',function(self)vars.bnet.duel=self:GetChecked()DCL:Register()end)
	DCLbox26:SetScript('PostClick',function(self)vars.bnet.pet=self:GetChecked()DCL:Register()end)
	DCLbox27:SetScript('PostClick',function(self)vars.bnet.guild=self:GetChecked()DCL:Register()end)
	DCLbox28:SetScript('PostClick',function(self)vars.bnet.group=self:GetChecked()DCL:Register()end)
	DCLbox29:SetScript('PostClick',function(self)vars.bnet.chart=self:GetChecked()DCL:Register()end)
	DCLbox30:SetScript('PostClick',function(self)vars.bnet.trade=self:GetChecked()DCL:Register()end)
	DCLbox31:SetScript('PostClick',function(self)vars.msg.duel=self:GetChecked()DCL:Register()end)
	DCLbox32:SetScript('PostClick',function(self)vars.msg.pet=self:GetChecked()DCL:Register()end)
	DCLbox33:SetScript('PostClick',function(self)vars.msg.guild=self:GetChecked()DCL:Register()end)
	DCLbox34:SetScript('PostClick',function(self)vars.msg.group=self:GetChecked()DCL:Register()end)
	DCLbox35:SetScript('PostClick',function(self)vars.msg.chart=self:GetChecked()DCL:Register()end)
	DCLbox36:SetScript('PostClick',function(self)vars.msg.trade=self:GetChecked()DCL:Register()end)
	DCLbox37:SetScript('PostClick',function(self)vars.dcl.msgs=self:GetChecked()DCL:Register()end)
	DCLbox38:SetScript('PostClick',function(self)vars.dcl.disable=self:GetChecked()DCL:Register()DCL:BTBGI()end)
	DCLPanel.okay=function()
		Decliner_SV=nil
		Decliner_SV=tcopy(vars)
		Decliner_SV.check=true
		notemp=true
	end
	DCLPanel.cancel=function()
		vars=tcopy(temp)
		DCL:Register()
		notemp=true
	end
	DCLPanel.default=function()
		vars=tcopy(defs)
		Decliner_SV=nil
		Decliner_SV=tcopy(defs)
		Decliner_SV.check=true
		DCL:SetCheckBoxes()
		DCL:BTBGI()
	end
	DCLPanel.refresh=function()
		DCL:SetCheckBoxes()
		DCL:BTBGI()
		if notemp then temp=tcopy(vars) notemp=false end
	end
	SLASH_DECLINER_1='/decliner'
	SLASH_DECLINER_2='/decline'
	SLASH_DECLINER_3='/dcl'
	SlashCmdList['DECLINER_']=function()InterfaceOptionsFrame_OpenToCategory(a)end
end

function DCL:SetCheckBoxes()
	DCLbox01:SetChecked(vars.enabled.duel)
	DCLbox02:SetChecked(vars.enabled.pet)
	DCLbox03:SetChecked(vars.enabled.guild)
	DCLbox04:SetChecked(vars.enabled.group)
	DCLbox05:SetChecked(vars.enabled.chart)
	DCLbox06:SetChecked(vars.enabled.trade)
	DCLbox07:SetChecked(vars.tell.duel)
	DCLbox08:SetChecked(vars.tell.pet)
	DCLbox09:SetChecked(vars.tell.guild)
	DCLbox10:SetChecked(vars.tell.group)
	DCLbox11:SetChecked(vars.tell.chart)
	DCLbox12:SetChecked(vars.tell.trade)
	DCLbox13:SetChecked(vars.guild.duel)
	DCLbox14:SetChecked(vars.guild.pet)
	DCLbox15:SetChecked(false)
	DCLbox16:SetChecked(vars.guild.group)
	DCLbox17:SetChecked(vars.guild.chart)
	DCLbox18:SetChecked(vars.guild.trade)
	DCLbox19:SetChecked(vars.friend.duel)
	DCLbox20:SetChecked(vars.friend.pet)
	DCLbox21:SetChecked(vars.friend.guild)
	DCLbox22:SetChecked(vars.friend.group)
	DCLbox23:SetChecked(vars.friend.chart)
	DCLbox24:SetChecked(vars.friend.trade)
	DCLbox25:SetChecked(vars.bnet.duel)
	DCLbox26:SetChecked(vars.bnet.pet)
	DCLbox27:SetChecked(vars.bnet.guild)
	DCLbox28:SetChecked(vars.bnet.group)
	DCLbox29:SetChecked(vars.bnet.chart)
	DCLbox30:SetChecked(vars.bnet.trade)
	DCLbox31:SetChecked(vars.msg.duel)
	DCLbox32:SetChecked(vars.msg.pet)
	DCLbox33:SetChecked(vars.msg.guild)
	DCLbox34:SetChecked(vars.msg.group)
	DCLbox35:SetChecked(vars.msg.chart)
	DCLbox36:SetChecked(vars.msg.trade)
	DCLbox37:SetChecked(vars.dcl.msgs)
	DCLbox38:SetChecked(vars.dcl.disable)
end